$Chip(6502)
$Title(Impossible Mission II)
$Subtitle(Put up simulated characters)
$Paginate
$Indent=8

**********************************************
*	Chart the characters onto bit_map    *
*	Entry parameters:		     *
*	    Expects Main memory selected     *
*	    Data is in text screen  (Main)   *
*	Self_modifying code (RAM only)	     *
**********************************************

	%include macro.inc
	%include prmseg.inc

	extern	lftlt,lftrt,lftlb,lftrb

	EXTERN	wr_aux, rd_aux, wr_main, rd_main
	EXTERN	char0, char1
	EXTERN	scanlo, scanhi
	extern	drkflg

	PUBLIC	chart, putlift, erlift

scr	equ	$60

	PUBLIC	lbflg

	SEG	PAGE0
	single	lbflg
	single	row
	single	column

	SEG	temps

	double	text_s
	double	bitmap
	single	shift
buf	ds	4

	SEG	maincode

chart	PROC

	move	row,#0

..loop0 sta	rd_aux
	ldy	row
	lda	scanlo,y
	sta	text_s
	sta	bitmap
	lda	scanhi,y
	ora	#$04
	sta	text_s+1
	sta	rd_main

	ldy	#0

..loop1 lda	(text_s),y
	and	#$3f
	beq	..nxcol

	jsr	putchar

..nxcol iny
	cpy	#40
	bcc	..loop1

	lda	row
	clc
	adc	#8
	sta	row
	cmp	#192
	bcc	..loop0

	rts

chart	ENDPROC

putchar PROC

	asl	a
	asl	a
	asl	a
	sta	..gt1+1
	sta	..gt2+1
	sta	..gt3+1
	sta	..gt4+1
	lda	#high(char0)/2
	rol	a
	sta	..gt1+2
	sta	..gt3+2
	and	#1
	ora	#high(char1)
	sta	..gt2+2
	sta	..gt4+2
***
	lda	text_s+1
	eor	#scr+4
	sta	bitmap+1

	tya
	and	#1
	bne	..odd

	ldx	#0

..loope move	buf+1,#0

..gt1	lda	char0,x
	sta	wr_aux
	sta	(bitmap),y
	sta	wr_main
	asl	a
..gt2	lda	char1,x
	rol	a
	sta	(bitmap),y	    ;buf+0
	rol	buf+1
	asl	a
	rol	buf+1

	iny
	sta	rd_aux
	sta	wr_aux
	lda	(bitmap),y
	and	#$7c
	ora	buf+1
	sta	(bitmap),y
	sta	rd_main
	sta	wr_main
	dey

	add	bitmap+1,4
	inx
	cpx	#8
	bcc	..loope

	rts

..odd	ldx	#0

..loopo

..gt4	lda	char1,x
	lsr	a
	sta	(bitmap),y
..gt3	lda	char0,x
	ror	a
	ror	buf+1
	lsr	a
	and	#$7c
	sta	buf+0
	ror	buf+1
	lsr	buf+1

	sta	rd_aux
	sta	wr_aux
	lda	(bitmap),y
	and	#$03
	ora	buf+0
	sta	(bitmap),y
	sta	rd_main
	sta	wr_main

	add	bitmap+1,4
	inx
	cpx	#8
	bcc	..loopo

	rts

putchar ENDPROC

**********************************************
*	Draw a lift routine...		     *
*   X: x coordinate, column		     *
*   Y: y coordinate, row		     *
**********************************************

putlift PROC

	stx	column
	tya
	asl	a
	asl	a
	asl	a
	sta	row

	clc
	adc	#8
	tax
	sta	rd_aux
	lda	scanlo,x
	sta	text_s
	sta	bitmap
	lda	scanhi,x
	clc
	adc	#$04
	sta	text_s+1
	sta	rd_main

	ldy	column
	lda	(text_s),y
	ora	#$c0
	sta	(text_s),y
	iny
	lda	(text_s),y
	ora	#$c0
	sta	(text_s),y
	iny
	lda	(text_s),y
	ora	#$c0
	sta	(text_s),y

	cpx	#200
	bcs	..noany
	cpx	#192
	bcs	..nobot

	lda	drkflg
	bne	..nobot

	ldy	column
	lda	(text_s),y
	and	#$1f
	tax
	lda	lftlb,x
	jsr	putchar
	iny

	lda	#4		; left bottom middle character
	jsr	putchar
	iny

	lda	#4		; right bottom middle character
	jsr	putchar
	iny

	lda	(text_s),y
	and	#$1f
	tax
	lda	lftrb,x
	jsr	putchar

..nobot sta	rd_aux
	ldx	row
	lda	scanlo,x
	sta	text_s
	sta	bitmap
	lda	scanhi,x
	ora	#$04
	sta	text_s+1
	sta	rd_main

	lda	drkflg
	bne	..noxxx

	ldy	column
	lda	(text_s),y
	and	#$1f
	tax
	lda	lftlt,x
	jsr	putchar
	iny

	lda	#24		; left top middle character
	jsr	putchar
	iny

	lda	#24		; right top middle character
	jsr	putchar
	iny

	lda	(text_s),y
	and	#$1f
	tax
	lda	lftrt,x
	jsr	putchar
..noxxx
	lda	#0
	sta	lbflg
	rts
..noany	lda	#$ff
	sta	lbflg
	rts

putlift ENDPROC

**********************************************
*   Erase a lift routine...		     *
*   A: 0=right, 1=down, -1=up		     *
*   X: x coordinate, column		     *
*   Y: y coordinate, row		     *
**********************************************

erlift	PROC

	sta	shift
	sty	row
	asl	row
	asl	row
	asl	row
	stx	column

	lda	row
	clc
	adc	#8
	tax
	sta	rd_aux
	lda	scanlo,x
	sta	text_s
	sta	bitmap
	lda	scanhi,x
	clc
	adc	#$04
	sta	text_s+1
	sta	rd_main

	ldy	column
	lda	(text_s),y
	and	#$3f
	sta	(text_s),y
	iny
	lda	(text_s),y
	and	#$3f
	sta	(text_s),y
	iny
	lda	(text_s),y
	and	#$3f
	sta	(text_s),y

	ldy	shift
	beq	..right
	dey
	beq	..up
	dey
	lbeq	..left

..down	ldx	row
	sta	rd_aux
	lda	scanlo,x
	sta	text_s
	sta	bitmap
	lda	scanhi,x
	ora	#$04
	sta	text_s+1
	sta	rd_main

..up	cpx	#192
	bcs	..done

	lda	drkflg
	bne	..done
	ldy	column

	lda	(text_s),y
	and	#$1f
	jsr	putchar
	iny

	lda	(text_s),y
	and	#$1f
	jsr	putchar
	iny

	lda	(text_s),y
	and	#$1f
	jsr	putchar
	iny

	lda	(text_s),y
	and	#$1f
	jsr	putchar

..done	rts

..right ldy	column
..lrcom
	cpx	#192
	bcs	..rt2

	lda	drkflg
	bne	..done

	lda	(text_s),y
	and	#$1f
	jsr	putchar

..rt2	ldx	row
	sta	rd_aux
	lda	scanlo,x
	sta	text_s
	sta	bitmap
	lda	scanhi,x
	ora	#$04
	sta	text_s+1
	sta	rd_main

	lda	drkflg
	bne	..done

	lda	(text_s),y
	and	#$1f
	jsr	putchar

	rts

..left	lda	column
	clc
	adc	#3
	tay
	jmp	..lrcom

erlift	ENDPROC

**********************************************
*   Erase a floor
*   X: x coordinate, column		     *
*   Y: y coordinate, row		     *
**********************************************
;
;erflr	PROC
;
;	sty	row
;	asl	row
;	asl	row
;	asl	row
;	stx	column
;
;	lda	row
;	clc
;	adc	#8
;	tax
;	sta	rd_aux
;	lda	scanlo,x
;	sta	text_s
;	sta	bitmap
;	lda	scanhi,x
;	ora	#$4
;	sta	text_s+1
;	sta	rd_main
;
;	ldy	column
;	lda	(text_s),y
;	jsr	putchar
;	iny
;	lda	(text_s),y
;	jsr	putchar
;	iny
;	lda	(text_s),y
;	jsr	putchar
;
;
;	ldx	row
;	sta	rd_aux
;	lda	scanlo,x
;	sta	text_s
;	sta	bitmap
;	lda	scanhi,x
;	ora	#$4
;	sta	text_s+1
;	sta	rd_main
;
;	ldy	column
;	lda	(text_s),y
;	jsr	putchar
;	iny
;	lda	(text_s),y
;	jsr	putchar
;	iny
;	lda	(text_s),y
;	jsr	putchar
;	iny
;	lda	(text_s),y
;	jsr	putchar
;
;	rts
;
;erflr	ENDPROC

	END
